@@ -87,14 +87,14 @@ york <- function(x){
 get.york.mswd <- function(x,a,b){
     xy <- get.york.xy(x,a,b)
     X2 <- 0
-    nn <- length(x[,'X'])
-    for (i in 1:nn){
+    ns <- length(x[,'X'])
+    for (i in 1:ns){
         E <- cor2cov2(x[i,'sX'],x[i,'sY'],x[i,'rXY'])
         X <- matrix(c(x[i,'X']-xy[i,1],x[i,'Y']-xy[i,2]),1,2)
         if (!any(is.na(X)))
             X2 <- X2 + 0.5*X %*% solve(E) %*% t(X)
     }
-    df <- (2*nn-2)
+    df <- ns-2
     out <- list()
     out$mswd <- as.numeric(X2/df)
     out$p.value <- as.numeric(1-stats::pchisq(X2,df))
@@ -181,3 +181,94 @@ data2york.PD <- function(x,exterr=FALSE,common=FALSE,...){
     colnames(out) <- c('X','sX','Y','sY','rXY')
     out
 }
+data2york.UThHe <- function(x,...){
+    ns <- length(x)
+    out <- matrix(0,ns,5)
+    colnames(out) <- c('X','sX','Y','sY','rXY')
+    R <- iratio('U238U235')
+    L8 <- lambda('U238')
+    L5 <- lambda('U235')
+    L2 <- lambda('Th232')
+    L7 <- lambda('Sm147')
+    f147 <- f147Sm()
+    P <- rep(0,ns)
+    sP <- rep(0,ns)
+    J <- matrix(0,1,9)
+    E <- matrix(0,9,9)
+    for (i in 1:ns){
+        P[i] <- 8*L8[1]*x[i,'U']*R[1]/(1+R[1]) +
+            7*L5[1]*x[i,'U']/(1+R[1]) +
+            6*L2[1]*x[i,'Th']
+        J[1,1] <- 8*L8[1]*R[1]/(1+R[1]) + 7*L5[1]/(1+R[1])  # dP.dU
+        J[1,2] <- 6*L2[1]                                   # dP.dTh
+        J[1,4] <- 8*x[i,'U']*R[1]/(1+R[1])                  # dP.dL8
+        J[1,5] <- 7*x[i,'U']/(1+R[1])                       # dP.dL5
+        J[1,6] <- 6*x[i,'Th']                               # dP.dL2
+        J[1,8] <- (8*L8[1]-7*L5[1])*x[i,'U']/(1+R[1])^2     # dP.dR
+        E[1,1] <- x[i,'errU']^2
+        E[2,2] <- x[i,'errTh']^2
+        E[4,4] <- L8[2]^2
+        E[5,5] <- L5[2]^2
+        E[6,6] <- L2[2]^2
+        E[7,7] <- L7[2]^2
+        E[8,8] <- R[2]^2
+        E[9,9] <- f147[2]^2
+        if (doSm(x)) {
+            P <- P + f147[1]*L7[1]*x[i,'Sm']
+            J[1,3] <- f147[1]*L7[1]       # dP.dSm
+            J[1,7] <- f147[1]*x[i,'Sm']   # dP.dL7
+            J[1,9] <- L7[1]*x[i,'Sm']     # dP.df147
+            E[3,3] <- x[i,'errSm']^2
+        }
+        sP[i] <- sqrt(J %*% E %*% t(J))
+    }
+    out[,'X'] <- P
+    out[,'sX'] <- sP
+    out[,'Y'] <- x[,'He']
+    out[,'sY'] <- x[,'errHe']
+    out
+}
+data2york.ThU <- function(x,type=2,...){
+    if (x$format %in% c(1,3) & type==1){
+        out <- x$x[,c('U238Th232','errU238Th232',
+                      'Th230Th232','errTh230Th232','rho')]
+    } else if (x$format %in% c(2,4) & type==2){
+        out <- x$x[,c('Th232U238','errTh232U238',
+                      'Th230U238','errTh230U238','rho')]
+    } else if (x$format %in% c(2,4) & type==1){
+        out <- ThConversionHelper(x)
+        colnames(out) <- c('U238Th232','errU238Th232',
+                           'Th230Th232','errTh230Th232','rho')
+    } else if (x$format %in% c(1,3) & type==2){
+        out <- ThConversionHelper(x)
+        colnames(out) <- c('Th232U238','errTh232U238',
+                           'Th230U238','errTh230U238','rho')
+    } else {
+        stop('Incorrect data format and/or plot type')
+    }
+    colnames(out) <- c('X','sX','Y','sY','rXY')
+    out
+}
+
+ThConversionHelper <- function(x){
+    ns <- length(x)
+    J <- matrix(0,2,2)
+    E <- matrix(0,2,2)
+    out <- matrix(0,ns,5)
+    for (i in 1:ns){
+        out[i,1] <- 1/x$x[i,1]
+        out[i,3] <- x$x[i,3]/x$x[i,1]
+        J[1,1] <- -out[i,1]/x$x[i,1]
+        J[2,1] <- -out[i,3]/x$x[i,1]
+        J[2,2] <- 1/x$x[i,1]
+        E[1,1] <- x$x[i,2]^2
+        E[2,2] <- x$x[i,4]^2
+        E[1,2] <- x$x[i,2]*x$x[i,4]*x$x[i,5]
+        E[2,1] <- E[1,2]
+        covmat <- J %*% E %*% t(J)
+        out[i,2] <- sqrt(covmat[1,1])
+        out[i,4] <- sqrt(covmat[2,2])
+        out[i,5] <- covmat[1,2]/(out[i,2]*out[i,4])
+    }
+    out
+}
